/*
 *  powctrld.c - daemon for Imagestation Power Control
 *
 *		Copyright (C) 1998-1999 Sony Corporation
 *		All Right Reserved
 *
 *	First Date: 1999/07/14
 *	Author: sinh@sm.sony.co.jp
 *
 *	$LOG$
 *
 */

// #include <stdio.h>
#include <stdlib.h>
#ifdef SYSV
#include <sys/types.h>
#endif

#include <utmp.h>
#include <ctype.h>
#include <string.h>
#include <setjmp.h>
#include <stdarg.h>

#include <sys/param.h>
#include <sys/errno.h>
#include <sys/ioctl.h>
#include <sys/stat.h>
#include <sys/wait.h>

#include <unistd.h>
#include <fcntl.h>
#include <sys/uio.h>
#include <sys/un.h>
#include <sys/time.h>
#include <sys/resource.h>
#include <signal.h>

#include <netinet/in.h>
#include <netdb.h>
#include <syscall.h>
#include <arpa/nameser.h>
#include <arpa/inet.h>
#include <resolv.h>

#if defined(__linux__)
#include <paths.h>
#endif

typedef unsigned short	vhalf;
typedef unsigned int	vword;

#include "pidfile.h"
#include "powctrl_ioctl.h"
#include "version.h"

#undef TRUE
#undef FALSE
#define TRUE	(1)
#define	FALSE	(0)

#define _PATH_LOGPID _PATH_VARRUN "powctrld.pid"
#define	POWCTRL_STATUS_POWEROFF_REQ	(1 << 10)

#define MAX_OFFSET 0x38
#define	PROGRAM		"powctrld"
#define	EXEC_CMD	"/sbin/init"
//#define	EXEC_CMD	"/export/home/sinh/driver/powctrld/init0"
#define	DEFAULT_SLEEP_TIME	(1)

void usage(void);
void show_version(void);
void set_sleep_time(char *s);
void set_debug(void);
void untty(void);
static void dbgprintf(char *, ...);
void die(int sig);

typedef struct argument_option {
	char	*optstr;	// argument option string
	char	*usage;		// usage
	unsigned int	n;	// number of argument
	void	(*func)();	// pointer to function
} ARG_OPTION;


char	*PidFile = _PATH_LOGPID;
char	*devname= "/dev/powctrl0";

char	*powargv[]= {
		EXEC_CMD,
		"0",
		NULL};
char	*powenvv[]= { 
		"PATH=/sbin:/usr/sbin:/usr/bin:/bin",
		NULL};
unsigned int	sleep_sec= DEFAULT_SLEEP_TIME;
unsigned int	Debug= FALSE;
unsigned int	devfd= -1;

ARG_OPTION option[] = {
	{ "-s", "[-s #]: sleep time inerval (sec)", 1, set_sleep_time},
	{ "-v", "[-v]:   display version", 0, show_version},
	{ "-d", "[-d]:   debug mode (not daemon)", 0, set_debug},
	{ "-h", "[-h]:   show this help", 0, usage},
	{ NULL, NULL, 0, NULL},
};


void usage(void)
{
    int i;

    fprintf( stderr, "usage: %s \n", PROGRAM);
    fprintf( stderr, "option: \n");;
    for( i= 0; option[i].optstr != NULL; i++)
	fprintf( stderr, "    %s\n", option[i].usage);

    exit(-1);
}

void show_version(void)
{
    printf( "%s %s-%s\n", PROGRAM, VERSION, PATCHLEVEL);
    exit(0);
}

void set_sleep_time(char *s)
{
    unsigned int	sec;

    sec= atoi(s);
    if (sec == 0)	sec= DEFAULT_SLEEP_TIME;

    sleep_sec= sec;
}


void set_debug(void)
{
    Debug= TRUE;
}


int main(int argc, char **argv)
{
    int	i, j;
    unsigned int	v=0;
    int	num_fds;
    int	quitpid = 0;

    if(argc > 1 ) {
	int	n;
	for( j= 0; option[j].optstr != NULL; j++);
	n= j;

	for( i= 1; i< argc; i++) {
	    for( j= 0; option[j].optstr != NULL; j++) {
		if (strcmp( argv[i], option[j].optstr)== 0) {
		    if (option[j].n) {
			(option[j].func)( argv[++i]);
		    } else
			(option[j].func)();
			
		    break;
		}
	    }
	    if (j == n)	usage();
	}
    }

#if 0
    daemon(0, 0);
#endif

    if ( !Debug ) {
	if (!check_pid(PidFile)) {
		quitpid = getpid();
		if (fork())
		{
			/* We try to wait the end of initialization */
			exit(0);
		}
		num_fds = getdtablesize();
		for (i= 0; i < num_fds; i++)
			(void) close(i);
		untty();
	}
	else {
		fputs( PROGRAM ": Already running.\n", stderr);
		exit(1);
	}
    }

    if ( !Debug ) {
	dbgprintf("Writing pidfile.\n");
	if (!check_pid(PidFile)) {
		if (!write_pid(PidFile)) {
			dbgprintf("Can't write pid.\n");
			exit(1);
		}
	}
	else {
		dbgprintf("Pidfile (and pid) already exist.\n");
		exit(1);
	}
    } /* if ( !Debug ) */

    (void) signal(SIGTERM, die);
    (void) signal(SIGINT, Debug ? die : SIG_IGN);
    (void) signal(SIGQUIT, Debug ? die : SIG_IGN);
    (void) signal(SIGCHLD, SIG_IGN);
//    (void) signal(SIGUSR1, Debug ? debug_switch : SIG_IGN);

    if (quitpid) {
	kill(quitpid, SIGINT);
    }

    if(( devfd = open( devname, O_RDONLY)) < 0) {
	fprintf(stderr,"Cannot open %s\n", devname);
	exit(-1);
    }

    while (1) {
	v = ioctl( devfd, POWCTRL_IOCPOWSTAT) & 0xffff;
	dbgprintf( "%s: current status [%x]\n", PROGRAM, v);
	if ( v != 0xffff && (v & POWCTRL_STATUS_POWEROFF_REQ)) {
		dbgprintf( "%s: exec %s \n", PROGRAM, EXEC_CMD);

		execv( EXEC_CMD, powargv);

		die(0);
	}

	sleep(sleep_sec);
    }

    return 0;
}

void untty()
#ifdef SYSV
{
    if ( !Debug ) {
	setsid();
    }
    return;
}

#else
{
    int i;

    if ( !Debug ) {
	i = open(_PATH_TTY, O_RDWR);
	if (i >= 0) {
		(void) ioctl(i, (int) TIOCNOTTY, (char *)0);
		(void) close(i);
	}
    }
}
#endif

static void dbgprintf(char *fmt, ...)

{
	va_list ap;

	if ( !Debug )
		return;

	va_start(ap, fmt);
	vfprintf(stdout, fmt, ap);
	va_end(ap);

	fflush(stdout);
	return;
}


void die(
	int sig
  )
{
	if (sig) {
		dbgprintf("%s: exiting on signal %d\n", PROGRAM, sig);
		errno = 0;
	}

	/* Clean-up files. */

	close(devfd);
	(void) remove_pid(PidFile);
	exit(0);
}

