/*
 *  powctrlmod.c -
 *
 *	Imagestation (PlayStation2) Power daemon Control
 *  
 *		Copyright (C) 1998-1999 Sony Corporation
 *		All Right Reserved
 *
 *	First Date: 1999/07/14
 *	Author: sinh@sm.sony.co.jp
 *
 *	$LOG$
 *
 */

#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ioctl.h>

typedef unsigned short	vhalf;
typedef unsigned int	vword;

#include "powctrl_ioctl.h"

#define MAX_OFFSET 0x38
#define	PROGRAM		"ps2powctrl"

const char *devname= "/dev/powctrl0";

void usage(
	char *prgm,
	char c
  )
{

    if (c)
	fprintf( stderr, "Unkown option string: t%c\n", c);

    fprintf( stderr, "usage: %s [-p] [-ts] [-tr] [-l <col>] \n", prgm);
    fprintf( stderr, "      -p  : goto shutdown process with power off.\n");
    fprintf( stderr, "      -ts : stop timer \n");
    fprintf( stderr, "      -tr : restart timer \n");
    fprintf( stderr, "      -d #     : change debug level to #.\n");
    fprintf( stderr, "      -l <col> : set color of Tool LED to <col>. \n");
    fprintf( stderr, "         <col> = {o,orange,g,green,m,mixed,f,off}\n");
    fprintf( stderr, "          g, green : Green\n");
    fprintf( stderr, "          bg       : Blink-Low Green\n");
    fprintf( stderr, "          hg       : Blink-High Green\n");
    fprintf( stderr, "          f        : LED Off\n");

    exit(-1);
}

int check_color(
	char *pgm,
	char *cols
  )
{
    int color= POWCTRL_TLED_GREEN;

    switch ( *cols ) {
    case 'g':
    case 'G':
		color= POWCTRL_WLED_GREEN;	break;
    case 'f':
		color= POWCTRL_WLED_OFF;	break;
    case 'b':
	cols++;
	switch ( *cols ) {
	case 'g':
	case 'G':
		color= POWCTRL_WLED_GREEN | POWCTRL_WLED_BLINK;
		break;
	default:
		usage( pgm, *cols);
	}
	break;

    case 'h':
	cols++;
	switch ( *cols ) {
	case 'g':
	case 'G':
		color= POWCTRL_WLED_GREEN | POWCTRL_WLED_BLINK_HIGH;
		break;
	default:
		usage( pgm, *cols);
	}
	break;

    default:
		usage( pgm, *cols);
    }

    return color;
}


int main(
	int argc,
	char **argv
  )
{
    extern char	*optarg;

    int	fd;
    int	ch, lvl, color= 0;
    char	*cols;
    POWCTRL_RWDATA	mode;

    int dbglvl= 0;
    int last= 0;
    int	poff= 0;
    int	tstop= 0;
    int tstrt= 0;
    int	led= 0;

    while ((ch= getopt(argc, argv, "pd:l:tsr")) != EOF) {
	switch( (char)ch) {
	case 'p':	// Power off
		poff= 1;
		break;

	case 'd':	// Debug level
		lvl= atoi(optarg);
		dbglvl= 1;
		break;

	case 'l':	// led control
		cols= optarg;
		if ((color= check_color( argv[0], cols))< 0)
			usage( argv[0], 0);
		led= 1;
		break;

	case 't':	// Timer
		break;

	case 's':	// Stop Timer
		if ((char)last == 't')	tstop= 1, tstrt= 0;
		else			usage( argv[0], ch);
		break;
		
	case 'r':	// Restart Timer
		if ((char)last == 't')	tstop= 0, tstrt= 1;
		else			usage( argv[0], ch);
		break;
		
	case '?':
	default:
		usage( argv[0], 0);
	}
	last= ch;
    }

    if(( fd = open( devname, O_RDONLY)) < 0) {
	fprintf(stderr,"Cannot open %s\n", devname);
	exit(-1);
    }

    if (dbglvl)
	ioctl( fd, POWCTRL_IOCPOWDBG, &lvl);

    if (led) {
	mode.value= (unsigned short)color;
	ioctl( fd, POWCTRL_IOC_TLED_COLOR, &mode);
    }

    if (tstop) {
	mode.value= POWCTRL_TIMER_DISABLE;
	ioctl( fd, POWCTRL_IOCTIMER, &mode);
    }

    if (tstrt) {
	mode.value= POWCTRL_TIMER_ENABLE;
	ioctl( fd, POWCTRL_IOCTIMER, &mode);
    }

    if (poff)	ioctl( fd, POWCTRL_IOCPOWEROFF);

    close(fd);

    return 0;
}
