/*
 *  powctrl_poweroff.c - Poweroff Assert for Imagestation Power Control
 *  
 *		Copyright (C) 1998-1999 Sony Corporation
 *		All Right Reserved
 *
 *	First Date: 1999/07/14
 *	Author: sinh@sm.sony.co.jp
 *
 *	$LOG$
 *
 */

#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ioctl.h>

typedef unsigned short	vhalf;
typedef unsigned int	vword;

#include "powctrl_ioctl.h"

#define MAX_OFFSET 0x38
#define	PROGRAM		"powctrl_poweroff"

const char *devname= "/dev/powctrl0";


void usage( char *prg)
{
   fprintf( stderr, "usage: %s [option]", prg);
   fprintf( stderr, "    [option]\n");
   fprintf( stderr, "        -f  : set led to off\n");
   fprintf( stderr, "        -g  : set led to green\n");
   fprintf( stderr, "        -bg : set led to low-blinking (green)\n");
   fprintf( stderr, "        -hg : set led to high-blinking (green)\n");

   exit(-1);
}


int main(int argc, char **argv)
{
    int	fd;
    POWCTRL_RWDATA	color;

    color.value= POWCTRL_TLED_GREEN;

    if ( argc > 1 ) {
	if (argv[1][0] == '-')
	    switch ( argv[1][1] ) {
	    case 'b':	// low-blinking mode
		switch ( argv[1][2] ) {
		case 'g':	// green
			color.value= POWCTRL_WLED_GREEN | 
					POWCTRL_WLED_BLINK;
			break;
		default:
			usage( argv[0]);
		}
		break;
	    case 'h':	// high-blinking mode
		switch ( argv[1][2] ) {
		case 'g':	// green
			color.value= POWCTRL_WLED_GREEN | 
					POWCTRL_WLED_BLINK_HIGH;
			break;
		default:
			usage( argv[0]);
		}
		break;
	    case 'f':	// led off
		color.value= POWCTRL_WLED_OFF;
		break;
	    case 'g':	// green
		color.value= POWCTRL_WLED_GREEN;
		break;
	    default:
		usage( argv[0]);
	    }
    }

    if(( fd = open( devname, O_RDONLY)) < 0) {
	fprintf(stderr,"Cannot open %s\n", devname);
	exit(-1);
    }

    ioctl( fd, POWCTRL_IOC_TLED_COLOR, &color);

    close(fd);

    return 0;
}
