/*
 *	nvramlinux.c
 *	nvram access interface for Linux
 *
 *			Copyright (C) 1999 Sony Corporation
 *			All Rights Reserved
 */


#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/fcntl.h>
#include <linux/nvram.h>
#include "setnvram.h"

#define NVRAM_DEVICE	"/dev/nvram"

static int nvram_fd;

int open_nvram(void)
{
    int size;

    if ((nvram_fd = open(NVRAM_DEVICE, O_RDWR)) < 0) {
	perror(NVRAM_DEVICE);
	return -1;
    }
    size = lseek(nvram_fd, 0, SEEK_END);
    lseek(nvram_fd, 0, SEEK_SET);

    return size;
}

int close_nvram(void)
{
    return close(nvram_fd);
}

int init_nvram(void)
{
    return ioctl(nvram_fd, NVRAM_INIT, 0);
}

int read_nvram(unsigned char *nvram, int size)
{
    lseek(nvram_fd, 0, SEEK_SET);
    if (read(nvram_fd, nvram, size) != size)
	return -1;

    /* erase checksum word */
    nvram[size - 1] = nvram[size - 2] = '\0';
    return 0;
}

int write_nvram(unsigned char *nvram, int size)
{
    lseek(nvram_fd, 0, SEEK_SET);
    /* force validate checksum */
    ioctl(nvram_fd, NVRAM_SETCKS, 0);

    if (write(nvram_fd, nvram, size) != size)
	return -1;
    return 0;
}

