/*
 * setnvram.c - set / show nvram contents
 *
 *			Copyright (C) 1999 Sony Corporation
 *			All Rights Reserved
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <linux/nvram.h>
#include "setnvram.h"

#define VAR_CMDLINE	"CMDLINE"

int nvram_size;
unsigned char *buf1, *buf2;
char **varp;
int iscmdline = 0;
char *cmd_name = "setnvram";

void setnvram(int argc, char **argv);
void usage(void);

int main(int argc, char **argv)
{
    if (strlen(argv[0]) >= 10 &&
	strcmp(argv[0] + strlen(argv[0]) - 10, "setcmdline") == 0) {
	iscmdline++;
	cmd_name = "setcmdline";
    }

    if ((nvram_size = open_nvram()) < 0)
	exit(1);

    if ((buf1 = (unsigned char *)malloc(nvram_size)) == NULL ||
	(buf2 = (unsigned char *)malloc(nvram_size)) == NULL ||
	(varp = (char **)malloc(nvram_size * sizeof(char *))) == NULL) {
    	perror(cmd_name);
	exit(1);
    }

    setnvram(argc, argv);

    close_nvram();
    return 0;
}

void setnvram(int argc, char **argv)
{
    int i;
    char *p;

    if (argc == 1) {
	if (read_nvram(buf1, nvram_size) < 0) {
	    fprintf(stderr, "%s: nvram checksum error\n", cmd_name);
	    exit(1);
	}
	split_var(buf1, varp);

	if (iscmdline) {
	    /* show only CMDLINE */
	    if ((p = get_var(VAR_CMDLINE, varp)) == NULL) {
		fprintf(stderr, "%s: no CMDLINE in nvram\n", cmd_name);
		exit(1);
	    }
	    printf("%s\n", p + strlen(VAR_CMDLINE) + 1);
	} else {
	    /* show all variables */
	    for (i = 0; varp[i] != NULL; i++)
		printf("%s\n", varp[i]);
	}
	return;
    }

    if (strcmp(argv[1], "-c") == 0) {
	if (iscmdline) {
	    /* nothing to do */
	} else {
	    /* initialize nvram */
	    init_nvram();
	    return;
	}
    } else if (argv[1][0] == '-') {
	usage();
    }

    /* set nvram variable(s) */

    if (read_nvram(buf1, nvram_size) < 0)
	memset(buf1, 0, nvram_size);
    split_var(buf1, varp);

    if (iscmdline) {
	p = (char *)malloc(strlen(VAR_CMDLINE) + 2);
	strcpy(p, VAR_CMDLINE "=");
	for (i = 1; i < argc; i++) {
	    if ((p = realloc(p, strlen(p) + strlen(argv[i]) + 2)) == NULL) {
		perror(cmd_name);
		exit(1);
	    }
	    strcat(p, argv[i]);
	    if (i + 1 != argc)
		strcat(p, " ");
	}
	if (strcmp(argv[1], "-c") == 0)
	    set_var(VAR_CMDLINE, varp);
	else
	    set_var(p, varp);
    } else {
	for (i = 1; i < argc; i++)
	    set_var(argv[i], varp);
    }

    if (pack_var(buf2, nvram_size, varp) < 0) {
	fprintf(stderr, "%s: out of nvram space\n", cmd_name);
	exit(1);
    }

    if (write_nvram(buf2, nvram_size) < 0) {
	perror(cmd_name);
	exit(1);
    }
}

void usage(void)
{
    if (iscmdline)
	fprintf(stderr, "\
usage:	setcmdline			: show CMDLINE
	setcmdline -c			: clear CMDLINE
	setcmdline args ...		: set CMDLINE
");
    else
	fprintf(stderr, "\
usage:	setnvram			: show all nvram contents
	setnvram -c			: clear nvram
	setnvram var[=value] ...	: set nvram variables
");
    exit(1);
}
